#include "Feedback.h"
#include "Wpilib.h"
#include <stdio.h>
#include <stdarg.h>
#include <vector>
#include <string>

#define USE_TWO_CAMERAS


static bool camTwoIsPresent()
{
	static const uInt32 MAX_CAMS = 10;
	IMAQdxCameraInformation camInfos[MAX_CAMS];
	uInt32 camCount = MAX_CAMS;
	IMAQdxError err = IMAQdxEnumerateCameras(camInfos, &camCount, 1);
	if (err != IMAQdxErrorSuccess)
		return false;
	for (uInt32 i = 0; i < camCount; ++i) {
		printf("Found camera %s\n", camInfos[i].InterfaceName);
		if (strcmp(camInfos[i].InterfaceName, "cam1") == 0)
			return true;
	}
	return false;
}


	Feedback::Feedback() : camServer(NULL), frontCam(true), hasBackCam(false)

	{
		camServer = ThunderCameraServer::GetInstance();
		std::vector<std::string> v;
		v.push_back("cam0");
#ifdef USE_TWO_CAMERAS
		hasBackCam = camTwoIsPresent();
		if (hasBackCam)
			v.push_back("cam1");
#endif
#ifndef USE_USBCAMERACLASS
		camServer->StartAutomaticCapture(v);
#else
		// defaults seem to be: Auto white bal; exposure 50%; brigthness 80%
		//
		// axis was: color level 70; brightness 50; sharpness 0; white balance Fixed Fluorescent 1; exposure Auto; no exposure priority
		std::vector<std::shared_ptr<USBCamera> > camVec;
		for (size_t i = 0; i < v.size(); ++i) {
			std::shared_ptr<USBCamera> camera =	std::make_shared<USBCamera>(v[i].c_str(), true);
			camera->OpenCamera();
			camera->SetExposureAuto();
			camera->SetBrightness(50);
			camera->SetWhiteBalanceManual(kFixedFluorescent1);
			camVec.push_back(camera);
		}
		camServer->StartAutomaticCapture(v);
#endif
	}

void Feedback::send_Debug_String(const char *subsystemName, const char *dataName, const char *format, ...) {//

	std::string keyName;
		keyName = subsystemName;
		keyName += "_";
		keyName += dataName; //adds a _ after your subsytem name, the name of your specific value after that
	  char buffer[256]; //sets the maximum character count to 256
	  va_list args; // args is the "..."
	  va_start (args, format); //marks the start of the list of arguments(args)
	  vsnprintf (buffer,256,format, args); //printing the args after the keyName
	  va_end (args);//marks end of arguments (this allows us to change the arguments and not the keyname)
	  SmartDashboard::PutString(keyName, buffer);//this prints keyName into the table, then the arguments
	}

void Feedback::send_Debug_Double(const char *subsystemName, const char *dataName, double format){

	std::string keyName;
	keyName = subsystemName;
	keyName += "_";
	keyName += dataName; // a _ after the subsystem name, and adds the name of the value after that
	SmartDashboard::PutNumber(keyName, format); //this prints keyName and the data into the table
}

void Feedback::swapCamera()
{
#ifdef USE_TWO_CAMERAS
	if (hasBackCam) {
        frontCam = !frontCam;
        camServer->SetCurrentCameraIndex(frontCam ? 0 : 1);
	}
#endif
}

bool Feedback::isFrontCamera()
{
	return (frontCam);
}

